package com.ruoyi.project.shiftmgr.service;

import com.ruoyi.project.shiftmgr.domain.TShiftLog;

import java.util.List;

/**
 * 轮班日志Service接口
 * 
 * @author ruoyi
 * @date 2023-08-17
 */
public interface ITShiftLogService 
{
    /**
     * 查询轮班日志
     * 
     * @param id 轮班日志主键
     * @return 轮班日志
     */
    public TShiftLog selectTShiftLogById(Long id);

    /**
     * 查询轮班日志列表
     * 
     * @param tShiftLog 轮班日志
     * @return 轮班日志集合
     */
    public List<TShiftLog> selectTShiftLogList(TShiftLog tShiftLog);

    /**
     * 新增轮班日志
     * 
     * @param tShiftLog 轮班日志
     * @return 结果
     */
    public int insertTShiftLog(TShiftLog tShiftLog);

    /**
     * 修改轮班日志
     * 
     * @param tShiftLog 轮班日志
     * @return 结果
     */
    public int updateTShiftLog(TShiftLog tShiftLog);

    /**
     * 批量删除轮班日志
     * 
     * @param ids 需要删除的轮班日志主键集合
     * @return 结果
     */
    public int deleteTShiftLogByIds(Long[] ids);

    /**
     * 删除轮班日志信息
     * 
     * @param id 轮班日志主键
     * @return 结果
     */
    public int deleteTShiftLogById(Long id);
}
